@@ -5,15 +5,12 @@
 # 2) adb forward tcp:2828 tcp:2828
 
 from optparse import OptionParser
-from StringIO import StringIO
 import os
 import pkg_resources
 import re
 import time
 import traceback
 from urlparse import urlparse
-import xml.dom.minidom
-import zipfile
 import sys
 
 from b2gpopulate import B2GPopulate
@@ -22,15 +19,16 @@ import dzclient
 import gaiatest
 from marionette import Actions
 from marionette import Marionette
+from marionette import Wait
+from marionette import expected
 from marionette.by import By
 from marionette.errors import MarionetteException
 from marionette.gestures import smooth_scroll
 import mozdevice
 import mozlog
+import mozversion
 import numpy
 
-from wait import MarionetteWait
-
 TEST_TYPES = ['startup', 'scrollfps']
 
 
@@ -90,50 +88,16 @@ class DatazillaPerfPoster(object):
         dm = mozdevice.DeviceManagerADB()
         self.device.add_device_manager(dm)
 
-        if self.device.is_android_build:
-            self.logger.debug('Getting gaia, gecko and build revisions')
-            for path in ['/system/b2g', '/data/local']:
-                path += '/webapps/settings.gaiamobile.org/application.zip'
-                if self.device.manager.fileExists(path):
-                    zip_file = zipfile.ZipFile(StringIO(
-                        self.device.manager.pullFile(path)))
-                    with zip_file.open('resources/gaia_commit.txt') as f:
-                        gaia_revision = f.read().splitlines()[0]
-                        self.logger.debug('Gaia revision: %s' % gaia_revision)
-                        self.ancillary_data['gaia_revision'] = gaia_revision
-                    break
-            else:
-                self.logger.warn('application.zip does not exist, perhaps '
-                                 'gaia has not been flashed to the device')
-            try:
-                sources_xml = sources and xml.dom.minidom.parse(sources) or \
-                    xml.dom.minidom.parseString(self.device.manager.catFile(
-                        'system/sources.xml'))
-                for element in sources_xml.getElementsByTagName('project'):
-                    path = element.getAttribute('path')
-                    revision = element.getAttribute('revision')
-                    if not self.ancillary_data.get('gaia_revision') and \
-                            path in 'gaia':
-                        self.ancillary_data['gaia_revision'] = revision
-                    if path in ['gecko', 'build']:
-                        self.ancillary_data['_'.join(
-                            [path, 'revision'])] = revision
-            except:
-                self.logger.warn('Unable to get revisions from sources')
-                pass
-
-            self.logger.debug('Getting build properties')
-            build_props = self.device.manager.pullFile('/system/build.prop')
-            desired_props = ('ro.build.version.incremental',
-                             'ro.build.version.release',
-                             'ro.build.date.utc')
-            for line in build_props.split('\n'):
-                if not line.strip().startswith('#') and '=' in line:
-                    name, value = [s.strip() for s in line.split('=', 1)]
-                    if name in desired_props:
-                        self.logger.debug('Build property: %s (%s)' % (
-                            name, value))
-                        self.ancillary_data[name] = value
+        version = mozversion.get_version(sources=sources, dm_type='adb')
+        self.ancillary_data['build_revision'] = version.get('build_changeset')
+        self.ancillary_data['gaia_revision'] = version.get('gaia_changeset')
+        self.ancillary_data['gecko_revision'] = version.get('gecko_changeset')
+        self.ancillary_data['ro.build.version.incremental'] = version.get(
+            'device_firmware_version_incremental')
+        self.ancillary_data['ro.build.version.release'] = version.get(
+            'device_firmware_version_release')
+        self.ancillary_data['ro.build.date.utc'] = version.get(
+            'device_firmware_date')
 
         self.required = {
             'gaia revision': self.ancillary_data.get('gaia_revision'),
@@ -145,7 +109,7 @@ class DatazillaPerfPoster(object):
             'branch': datazilla_config['branch'],
             'oauth key': datazilla_config['oauth_key'],
             'oauth secret': datazilla_config['oauth_secret'],
-            'machine name': mac_address or 'unknown',
+            'machine name': datazilla_config['machine_name'] or mac_address,
             'device name': datazilla_config['device_name'],
             'os version': settings.get('deviceinfo.os'),
             'id': settings.get('deviceinfo.platform_build_id')}
@@ -203,6 +167,7 @@ class B2GPerfRunner(DatazillaPerfPoster):
         self.settle_time = kwargs.pop('settle_time')
         self.test_type = kwargs.pop('test_type')
         self.testvars = kwargs.pop('testvars', {})
+        self.reset = kwargs.pop('reset')
 
         DatazillaPerfPoster.__init__(self, *args, **kwargs)
         # Add various attributes to the report
@@ -251,7 +216,8 @@ class B2GPerfRunner(DatazillaPerfPoster):
 
             test = test_class(self.marionette, app_name, self.logger,
                               self.iterations, self.delay, self.device,
-                              self.restart, self.settle_time, self.testvars)
+                              self.restart, self.settle_time, self.testvars,
+                              self.reset)
             try:
                 test.run()
 
@@ -278,7 +244,7 @@ class B2GPerfRunner(DatazillaPerfPoster):
 class B2GPerfTest(object):
 
     def __init__(self, marionette, app_name, logger, iterations, delay,
-                 device, restart, settle_time, testvars):
+                 device, restart, settle_time, testvars, reset):
         self.marionette = marionette
         self.app_name = app_name
         self.logger = logger
@@ -288,6 +254,7 @@ class B2GPerfTest(object):
         self.restart = restart
         self.settle_time = settle_time
         self.testvars = testvars
+        self.reset = reset
         self.requires_connection = False
         self.b2gpopulate = B2GPopulate(self.marionette)
 
@@ -314,6 +281,18 @@ class B2GPerfTest(object):
             self.logger.debug('Stopping B2G')
             self.device.stop_b2g()
 
+        if self.reset:
+            self.logger.debug('Removing persistent storage')
+            self.device.manager.removeDir('/data/local/storage/persistent')
+            self.device.manager.removeDir('/data/local/indexedDB')
+
+            self.logger.debug('Removing profile')
+            self.device.manager.removeDir('/data/b2g/mozilla')
+
+            self.logger.debug('Removing files from sdcard')
+            for item in self.device.manager.listFiles('/sdcard/'):
+                self.device.manager.removeDir('/'.join(['/sdcard', item]))
+
         self.logger.debug('Populating databases')
         self.populate_databases()
 
@@ -340,7 +319,7 @@ class B2GPerfTest(object):
         self.data_layer.set_setting('keyboard.ftu.enabled', False)
 
         self.logger.debug('Unlocking device')
-        gaiatest.LockScreen(self.marionette).unlock()
+        self.device.unlock()
 
         self.logger.debug('Killing all running apps')
         self.apps.kill_all()
@@ -499,7 +478,7 @@ class B2GPerfScrollTest(B2GPerfTest):
 
         self.scroll()
 
-        MarionetteWait(self.marionette, 30).until(
+        Wait(self.marionette, timeout=30).until(
             lambda m: m.execute_script(
                 'return window.wrappedJSObject.touchend;', new_sandbox=False))
 
@@ -539,7 +518,7 @@ class B2GPerfScrollBrowserTest(B2GPerfScrollTest):
         app.go_to_url('http://taskjs.org/')
         # TODO Move readyState wait into app object
         app.switch_to_content()
-        MarionetteWait(self.marionette, 30).until(
+        Wait(self.marionette, timeout=30).until(
             lambda m: m.execute_script(
                 'return window.document.readyState;',
                 new_sandbox=False) == 'complete')
@@ -564,9 +543,10 @@ class B2GPerfScrollContactsTest(B2GPerfScrollTest):
     def before_scroll(self):
         B2GPerfScrollTest.before_scroll(self)
         self.logger.debug('Waiting for contacts to be displayed')
-        MarionetteWait(self.marionette, 240).until(
-            lambda m: m.find_element(
-                *self.contacts._contact_locator).is_displayed())
+        contact = Wait(self.marionette, timeout=240).until(
+            expected.element_present(*self.contacts._contact_locator))
+        Wait(self.marionette, timeout=30).until(
+            expected.element_displayed(contact))
 
     def populate_databases(self):
         self.b2gpopulate.populate_contacts(self.contact_count, restart=False)
@@ -587,9 +567,8 @@ class B2GPerfScrollEmailTest(B2GPerfScrollTest):
     def before_scroll(self):
         B2GPerfScrollTest.before_scroll(self)
         self.logger.debug('Waiting for emails to be displayed')
-        MarionetteWait(self.marionette, 30).until(
-            lambda m: m.find_element(
-                By.CLASS_NAME, 'msg-header-author').is_displayed())
+        Wait(self.marionette, timeout=30).until(expected.element_displayed(
+            self.marionette.find_element(By.CLASS_NAME, 'msg-header-author')))
 
     def scroll(self):
         # TODO Needs updating/fixing once we can pre-populate emails
@@ -597,7 +576,7 @@ class B2GPerfScrollEmailTest(B2GPerfScrollTest):
             By.CLASS_NAME, 'msg-header-author')
         # We're dynamically adding these elements from a template, and the
         # first one found is blank.
-        MarionetteWait(self.marionette, 30).until(
+        Wait(self.marionette, timeout=30).until(
             lambda m: emails[0].get_attribute('innerHTML'))
         emails = self.marionette.find_elements(
             By.CLASS_NAME, 'msg-header-author')
@@ -620,13 +599,12 @@ class B2GPerfScrollGalleryTest(B2GPerfScrollTest):
         self.logger.debug('Sleep for 5 seconds to allow scan to start')
         time.sleep(5)
         self.logger.debug('Waiting for correct number of pictures')
-        MarionetteWait(self.marionette, 240).until(
+        Wait(self.marionette, timeout=240).until(
             lambda m: len(m.find_elements(
                 *self.gallery._gallery_items_locator)) == self.picture_count)
         self.logger.debug('Waiting for progress bar to be hidden')
-        MarionetteWait(self.marionette, 60).until_not(
-            lambda m: m.find_element(
-                *self.gallery._progress_bar_locator).is_displayed())
+        Wait(self.marionette, timeout=60).until(expected.element_not_displayed(
+            self.marionette.find_element(*self.gallery._progress_bar_locator)))
 
     def populate_files(self):
         self.b2gpopulate.populate_pictures(self.picture_count)
@@ -667,14 +645,14 @@ class B2GPerfScrollHomescreenTest(B2GPerfScrollTest):
                 page.size['width'] / 2,
                 page.size['width'] / 100 * 10,
                 page.size['width'] / 2, 200).perform()
-            MarionetteWait(self.marionette, 30).until(
+            Wait(self.marionette, timeout=30).until(
                 lambda m: page.get_attribute('aria-hidden') or
                 not page.is_displayed())
         for page in reversed(self.marionette.find_elements(
                 By.CSS_SELECTOR, '#icongrid > div')[1:]):
-            MarionetteWait(self.marionette, 30).until_not(
-                lambda m: page.get_attribute('aria-hidden') or
-                not page.is_displayed())
+            Wait(self.marionette, timeout=30).until(
+                lambda m: page.is_displayed() or
+                not page.get_attribute('aria-hidden'))
             self.logger.debug('Swiping to previous page of apps')
             action.flick(
                 page,
@@ -689,9 +667,10 @@ class B2GPerfScrollMessagesTest(B2GPerfScrollTest):
     def before_scroll(self):
         B2GPerfScrollTest.before_scroll(self)
         self.logger.debug('Waiting for messages to be displayed')
-        MarionetteWait(self.marionette, 240).until(
-            lambda m: m.find_element(
-                By.CSS_SELECTOR, '#threads-container li').is_displayed())
+        Wait(self.marionette, timeout=240).until(
+            expected.element_displayed(
+                self.marionette.find_element(
+                    By.CSS_SELECTOR, '#threads-container li')))
 
     def populate_databases(self):
         self.b2gpopulate.populate_messages(200, restart=False)
@@ -722,8 +701,9 @@ class B2GPerfScrollMusicTest(B2GPerfScrollTest):
         self.logger.debug('Sleep for 5 seconds to allow scan to start')
         time.sleep(5)
         self.logger.debug('Waiting for progress bar to be hidden')
-        MarionetteWait(self.marionette, 240).until_not(
-            lambda m: m.find_element(By.ID, 'scan-progress').is_displayed())
+        Wait(self.marionette, timeout=240).until(
+            expected.element_not_displayed(
+                self.marionette.find_element(By.ID, 'scan-progress')))
 
     def populate_files(self):
         self.b2gpopulate.populate_music(
@@ -768,13 +748,13 @@ class B2GPerfScrollVideoTest(B2GPerfScrollTest):
         self.logger.debug('Sleep for 5 seconds to allow scan to start')
         time.sleep(5)
         self.logger.debug('Waiting for correct number of videos')
-        MarionetteWait(self.marionette, 120).until(
+        Wait(self.marionette, timeout=120).until(
             lambda m: len(m.find_elements(
                 By.CSS_SELECTOR,
                 '#thumbnails .thumbnail')) == self.video_count)
         self.logger.debug('Waiting for progress bar to be hidden')
-        MarionetteWait(self.marionette, 60).until_not(
-            lambda m: m.find_element(By.ID, 'throbber').is_displayed())
+        Wait(self.marionette, timeout=60).until(expected.element_not_displayed(
+            self.marionette.find_element(By.ID, 'throbber')))
 
     def populate_files(self):
         self.b2gpopulate.populate_videos(self.video_count)
@@ -819,6 +799,11 @@ class dzOptionParser(OptionParser):
                         dest='datazilla_key',
                         metavar='str',
                         help='oauth key for datazilla server')
+        self.add_option('--dz-machine',
+                        action='store',
+                        dest='datazilla_machine_name',
+                        metavar='str',
+                        help='datazilla machine name')
         self.add_option('--dz-secret',
                         action='store',
                         dest='datazilla_secret',
@@ -842,6 +827,7 @@ class dzOptionParser(OptionParser):
             'host': datazilla_url.hostname,
             'project': options.datazilla_project,
             'branch': options.datazilla_branch,
+            'machine_name': options.datazilla_machine_name,
             'device_name': options.datazilla_device_name,
             'oauth_key': options.datazilla_key,
             'oauth_secret': options.datazilla_secret}
@@ -897,7 +883,14 @@ def cli():
                       action='store',
                       dest='testvars',
                       metavar='str',
-                      help='path to a json file with any test data required')
+                      help='path to a json file with any test data required'),
+    parser.add_option('--reset',
+                      action='store_true',
+                      dest='reset',
+                      default=False,
+                      help='reset the target to a clean state between tests '
+                           '(requires restart). WARNING: any personal data '
+                           'will be removed!')
     options, args = parser.parse_args()
 
     if not args:
@@ -922,6 +915,9 @@ def cli():
         with open(options.testvars) as f:
             testvars = json.loads(f.read())
 
+    if options.reset and not options.restart:
+        raise B2GPerfError('--reset requires restart')
+
     datazilla_config = parser.datazilla_config(options)
 
     # TODO command line option for address
@@ -936,7 +932,8 @@ def cli():
                             restart=options.restart,
                             settle_time=options.settle_time,
                             test_type=options.test_type,
-                            testvars=testvars)
+                            testvars=testvars,
+                            reset=options.reset)
     b2gperf.measure_app_perf(args)
 
 
